## $Id: student.py 18101 2025-06-30 11:56:08Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container for the various objects owned by students.
"""
import grok
from zope.interface import implementedBy
from zope.component import getUtility
from waeup.kofa.interfaces import (
    CLEARANCE, REQUESTED, GRADUATED, TRANSREL, TRANSVAL)
from waeup.kofa.utils.helpers import attrs_to_fields
from waeup.kofa.students.student import StudentFactory
from waeup.kofa.students.interfaces import IStudentNavigation, IStudentsUtils
from waeup.kofa.utils.helpers import get_current_principal
from kofacustom.nigeria.students.student import NigeriaStudent
from waeup.uniben.students.interfaces import (
    ICustomStudent, ICustomStudentPersonalEdit,
    IMedicalHistory, ITiship, INYSC)


class CustomStudent(NigeriaStudent):
    """This is a student container for the various objects
    owned by students.
    """
    grok.implements(
        ICustomStudent, IStudentNavigation, ICustomStudentPersonalEdit,
        IMedicalHistory, ITiship, INYSC)
    grok.provides(ICustomStudent)

    @property
    def transcript_enabled(self):
        #user = get_current_principal()
        #if user.id in ('admin', 'isouaba', 'med', 'zope.mgr'):
        #    return True
        final_clearance_enabled = getUtility(
            IStudentsUtils).final_clearance_enabled(self)
        if not final_clearance_enabled:
            return False
        if self.current_mode != 'ug_ft':
            return False
        if self.faccode in ('EDU', 'MED', 'DEN'):
            return False
        return True

    @property
    def is_jupeb(self):
        if self.current_mode == 'found':
            return True
        return 

    @property
    def clearance_locked(self):
        return self.state not in (CLEARANCE, )

    @property
    def studycourse_locked(self):
        # return self.state in (GRADUATED, TRANSREL, TRANSVAL)
        return self.state in (TRANSREL, TRANSVAL)

    @property
    def eligible_for_nysc(self):
        if not self.current_mode.startswith('ug') \
            and not self.current_mode.startswith('de'):
            return False
        studycourse = self['studycourse']
        certificate = getattr(studycourse,'certificate',None)
        current_level = studycourse.current_level
        end_level = certificate.end_level
        if not self.current_level >= end_level:
            return False
        return True


# Set all attributes of Student required in IStudent as field
# properties. Doing this, we do not have to set initial attributes
# ourselves and as a bonus we get free validation when an attribute is
# set.
CustomStudent = attrs_to_fields(CustomStudent)

class CustomStudentFactory(StudentFactory):
    """A factory for students.
    """

    def __call__(self, *args, **kw):
        return CustomStudent()

    def getInterfaces(self):
        return implementedBy(CustomStudent)